/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <package_manager.h>
#include "data.h"
#include "$(appName).h"

static bool __is_app_successful(app_manager_error_e err_code);
static bool __app_manager_app_info_cb(app_info_h app_info, void *user_data);

/**
 * @brief Function acquires the information structures of all the installed applications.
 * The acquired structures are packed into the list and returned.
 * For all the installed applications information acquisition, the app_manager_foreach_app_info()
 * function is used.
 * @return If the function succeeds, the the list of information structures of all installed
 * applications is returned. On failure NULL is returned.
 */
Eina_List *get_all_apps(void)
{
	Eina_List *app_list = NULL;
	app_manager_foreach_app_info(__app_manager_app_info_cb, &app_list);
	return app_list;
}

/**
 * @brief Function frees all the memory allocated for the applications list.
 * @param[in] app_list The applications list to be freed.
 * @return Function returns 'true' on success, otherwise 'false' is returned.
 */
bool destroy_all_apps_list(Eina_List *app_list)
{
	Eina_List *it = NULL;
	app_info_h *app_info = NULL;
	EINA_LIST_FOREACH(app_list, it, app_info)
		if (__is_app_successful(app_info_destroy(*app_info)))
			return true;

	return false;
}

/**
 * @brief Function obtains the application's label.
 * @param[in] app_info The application information handle.
 * @param[out] label The label acquired from the app_info handle.
 * @return Function returns 'true' on success, otherwise 'false' is returned.
 */
bool get_app_label(app_info_h *app_info, char **label)
{
	return __is_app_successful(app_info_get_label(*app_info, label));
}

/**
 * @brief Function obtains the application's icon path.
 * @param[in] app_info The application information handle.
 * @param[out] path The icon's path acquired from the app_info handle.
 * @return Function returns 'true' on success, otherwise 'false' is returned.
 */
bool get_app_icon(app_info_h *app_info, char **path)
{
	return __is_app_successful(app_info_get_icon(*app_info, path));
}

/**
 * @brief Function obtains the application's package name.
 * @param[in] app_info The application information handle.
 * @param[out] package The package name acquired from the app_info handle.
 * @return Function returns 'true' on success, otherwise 'false' is returned.
 */
bool get_app_package(app_info_h *app_info, char **package)
{
	return __is_app_successful(app_info_get_package(*app_info, package));
}

/**
 * @brief Internal helper function which validates the Application API functions result.
 * @param[in] err_code The error code returned by the Application API function.
 * @return Function returns 'true' if the input parameter is APP_MANAGER_ERROR_NONE,
 * otherwise 'false' is returned.
 */
static bool __is_app_successful(app_manager_error_e err_code)
{
	if (err_code == APP_MANAGER_ERROR_NONE)
		return true;

	dlog_print(DLOG_ERROR, LOG_TAG, "App manager failed. Err = %d (%s)", err_code, get_error_message(err_code));

	return false;
}

/**
 * @brief The callback function invoked on installed applications enumeration.
 * Its responsibility is to create the list of all the installed applications information
 * structures.
 * This function is attached by the app_manager_foreach_app_info() within get_all_apps()
 * function.
 * @param[in] app_info The application information handle.
 * @param[in] user_data The data passed to the app_manager_foreach_app_info() function.
 * @return Function returns 'true' on success, otherwise 'false' is returned.
 */
static bool __app_manager_app_info_cb(app_info_h app_info, void *user_data)
{
	bool nodisplay;
	__is_app_successful(app_info_is_nodisplay(app_info, &nodisplay));
	if (!nodisplay) {
		Eina_List **app_list = user_data;
		app_info_h *clone = malloc(sizeof(app_info_h));
		__is_app_successful(app_info_clone(clone, app_info));
		*app_list = eina_list_append(*app_list, clone);
	}
	return true;
}
